/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include "tcpip.h"

#define FBM_NORMAL	0
#define FBM_ALL		1
#define FBM_ANY		2
#define FBM_NOT		3

struct frag_node {
	u_int8_t mode;
	u_int16_t fragbits;
	int negate;
	struct alert *alert;
	struct detect *result;
	struct frag_node *next;
};

struct frag_detect {
	struct detect d;
	struct frag_node *nodes;
	struct detect *any;
};

/* Mask off the bits we need in the frag_off field */
static u_int16_t fbmask=__constant_htons(~IP_OFFMASK);

static void frag_match(struct detect *d, struct packet *p, int l)
{
	struct frag_detect *x = (struct frag_detect *)d;
	struct frag_node *n;
	u_int8_t frag;

	frag = p->layer[l+1].h.ip->frag_off & fbmask;

	for(n=x->nodes; n; n=n->next ) {
		int ret=0;

		switch ( n->mode ) {
		case FBM_NORMAL:
			ret = n->negate ^
				(frag == n->fragbits);
			break;
		case FBM_ANY:
			ret = n->negate ^
				((frag & n->fragbits) != 0);
			break;
		case FBM_ALL:
			ret = n->negate ^
				((frag & n->fragbits) == n->fragbits);
			break;
		case FBM_NOT:
			ret = n->negate ^
				((frag & n->fragbits) == 0);
			break;
		}

		if ( ret ) {
			if ( n->alert )
				alert_tag(p, n->alert, l);
			if ( n->result )
				n->result->match(n->result, p, l);
		}
	}

	if ( x->any )
		x->any->match(x->any, p, l);
}

static struct detect *frag_new(struct matcher *m)
{
	struct frag_detect *x;

	x = calloc(1, sizeof(*x));
	if ( x == NULL )
		return NULL;

	x->d.match = frag_match;

	return (struct detect *)x;
}

static struct detect **frag_default(struct detect *d)
{
	struct frag_detect *x = (struct frag_detect *)d;
	return &x->any;
}

static struct detect **frag_insert(struct detect *d, struct sig_node *n)
{
	struct frag_detect *x = (struct frag_detect *)d;
	struct frag_node *p = (struct frag_node *)n->private;
	struct frag_node *tmp;

	p->alert = n->alert;
	p->negate = n->negate;

	for(tmp=x->nodes; tmp; tmp=tmp->next) {
		if ( p->fragbits != tmp->fragbits )
			continue;
		if ( p->mode != tmp->mode )
			continue;
		if ( p->negate != tmp->negate )
			continue;

		tmp->alert = alert_best(tmp->alert, p->alert);
		n->match->free_priv(n->private);
		return &tmp->result;
	}

	p->next = x->nodes;
	x->nodes = p;

	return &p->result;
}

static int frag_validate(struct matcher *matcher, char *args,
			void **priv, struct modifier *m)
{
	struct frag_node *fb;
	char *tmp;

	if ( !(fb=calloc(1, sizeof(*fb))) ) {
		return 0;
	}

	for(tmp=args; *tmp; tmp++) {
		switch(*tmp) {
			case 'd':
			case 'D':
				fb->fragbits |= IP_DF;
				break;
			case 'm':
			case 'M':
				fb->fragbits |= IP_MF;
				break;
			case 'r':
			case 'R':
			case 'c':
			case 'C':
				fb->fragbits |= IP_CE;
				break;
			case '!':
				fb->mode=FBM_NOT;
				break;
			case '+':
				fb->mode=FBM_ANY;
				break;
			case '*':
				fb->mode=FBM_ALL;
				break;
			case ' ':
			case '\t':
			case ',':
				break;
			default:
				free(fb);
				return 0;
		}
	}

	fb->fragbits = htons(fb->fragbits);
	*priv = fb;
	return 1;
}

struct matcher ip_fragbits_matcher = {
	.validate = frag_validate,
	.new_node = frag_new,
	.insert_node = frag_insert,
	.insert_default_node = frag_default,
	.free_priv = free,
};
