/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __ALERT_HEADER_INCLUDED__
#define __ALERT_HEADER_INCLUDED__

#define RATE_SEC (100UL)
#define RATE_MIN (60UL*RATE_SEC)
#define RATE_HR  (60UL*RATE_MIN)
#define RATE_DAY (24UL*RATE_HR)

#include <limits.h>
#define ALERT_BUILTIN INT_MAX

struct tokenbucket {
	unsigned long toks; /* tokens */
	unsigned long last_msg; /* time of last msg */
	unsigned long cost; /* max rate */
	unsigned long burst; /* burst */
	int missed; /* number of missed alerts */
};

#define init_generator(x,y) {.name = x, .t = y}
struct generator {
	const char *name;
	struct tokenbucket *t;
};

#define TAG_TYPE_SESSION	0x0
#define TAG_TYPE_HOST		0x1

#define TAG_METRIC_NONE		0x0
#define TAG_METRIC_PACKETS	0x1
#define TAG_METRIC_SECONDS	0x2
#define TAG_METRIC_BYTES	0x3

#define TAG_DIR_SRC		0x0 /* To or from SRC */
#define TAG_DIR_DST		0x1 /* To or from DST */
#define TAG_DIR_SRC_ONLY	0x2 /* From SRC */
#define TAG_DIR_DST_ONLY	0x3 /* To DST */
#define TAG_DIR_SERVER		0x4 /* From server */
#define TAG_DIR_CLIENT		0x5 /* From client */
#define TAG_DIR_BOTH		0x6 /* From DST to SRC */
#define TAG_DIR_EITHER		0x7 /* To or from DST or SRC */

struct alert_tag {
	unsigned int type;
	unsigned int metric;
	unsigned int count;
	unsigned int direction;
};

struct alert {
	char		*alert;
	u_int32_t	sid;
	u_int32_t	rev;
	u_int8_t	priority;

	/* Rate limiting */
	struct tokenbucket t;
	struct generator *gen;

	/* For rule ordering */
	int order;

	/* Alert tagging */
	struct alert_tag *tag;
};

void alert(struct packet *) __nonull(1);

#endif /* __ALERT_HEADER_INCLUDED__ */
