/*
 * This file is part of dotscara
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 */
#ifndef __INITMEM_HEADER_INCLUDED__
#define __INITMEM_HEADER_INCLUDED__

/* for memset */
#include <string.h>

/** initmem_slab - Initmem memory area descriptor */
struct initmem_slab {
	/** Amount of data remaining in this area */
	size_t len;
	/** Pointer to the remaining free data */
	void *ptr;
	/** Mext memory area in the list */
	struct initmem_slab *next;
};

/** initmem - Initmem object */
struct initmem {
	/** Size to make system memory requests */
	size_t slab_size;
	/** Head of memory area list */
	struct initmem_slab *head;
	/** Tail of memory area list */
	struct initmem_slab *tail;
};

int __public initmem_new(struct initmem *m, size_t slab_size)
	__nonull(1);
void __public initmem_free(struct initmem *m) __nonull(1);
void __public *__initmem_alloc_slow(struct initmem *m, size_t sz)
	__nonull(1) __malloc;
static inline void *initmem_alloc(struct initmem *m, size_t sz)
	__nonull(1) __malloc;
static inline void *initmem_alloc_zero(struct initmem *m, size_t sz)
	__nonull(1) __malloc;

static inline void *initmem_alloc(struct initmem *m, size_t sz)
{
	void *ret;
	void *end;

	if ( unlikely(m->tail == NULL || m->tail->len < sz) ) {
		return __initmem_alloc_slow(m, sz);
	}

	ret = m->tail->ptr;
	end = m->tail->ptr + m->tail->len;
	m->tail->ptr += sz;

	if ( m->tail->ptr > end )
		m->tail->len = 0;
	else
		m->tail->len -= sz;

	return ret;
}

static inline void *initmem_alloc_zero(struct initmem *m, size_t sz)
{
	void *ret;

	ret = initmem_alloc(m, sz);
	if ( ret != NULL )
		memset(ret, 0, sz);

	return ret;
}

#endif /* __INITMEM_HEADER_INCLUDED__ */
