/*
 * This file is part of dotscara
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 */
#ifndef __MPOOL_HEADER_INCLUDED__
#define __MPOOL_HEADER_INCLUDED__

/* for memset */
#include <string.h>

/** mpool descriptor */
struct mpool {
	/** Object size */
	size_t obj_size;
	/** Size of each block including mpool_hdr overhead */
	size_t slab_size;
	/** List of blocks */
	struct mpool_hdr *slabs;
};

/** mpool_hdr - mpool memory area descriptor */
struct mpool_hdr {
	/** Pointer to next item in the list */
	struct mpool_hdr *next;
	/** Pointer to next available object */
	void *next_obj;
	/** Data up to the slab_size */
	u_int8_t data[0];
};

int __public mpool_new(struct mpool *m, size_t obj_size, size_t slab_size)
	__nonull(1);
void __public mpool_free(struct mpool *m) __nonull(1);
void __public mpool_destroy(struct mpool *m, void(*dtor)(void *)) __nonull(1,2);
void __public *__mpool_alloc_slow(struct mpool *m) __nonull(1);
static inline void *mpool_alloc_zero(struct mpool *m) __malloc __nonull(1);
static inline void *mpool_alloc(struct mpool *m) __malloc __nonull(1);


/* Fast path public API */
static inline void *mpool_alloc(struct mpool *m)
{
	if ( m->slabs != NULL ) {
		void *ret = m->slabs->next_obj;
		m->slabs->next_obj += m->obj_size;

		if ( m->slabs->next_obj <= ((void *)m->slabs) + m->slab_size )
			return ret;
	}

	return __mpool_alloc_slow(m);
}

/* Same as mpool_alloc, but zeroed */
static inline void *mpool_alloc_zero(struct mpool *m)
{
	void *ret;

	ret = mpool_alloc(m);
	if ( ret )
		memset(ret, 0, m->obj_size);

	return ret;
}

#endif /* __MPOOL_HEADER_INCLUDED__ */
