/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __FIRESTORM_HEADER_INCLUDED__
#define __FIRESTORM_HEADER_INCLUDED__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#include <dotscara/dotscara.h>
#include <dotscara/sys.h>

struct fvec {
	void *ptr;
	size_t len;
};

#include <packet.h>
#include <notifier.h>

#define M_UNSET	0 /* Unclassified */
#define M_CRIT	1 /* Service/data is lost */
#define M_ERR	2 /* We can't do something you asked for */
#define M_WARN	3 /* We can work around this, but you should know */
#define M_INFO	4 /* Informational notices */
#define M_DEBUG	5 /* For developers */
#define M_MAX	6

/* Subsystem intialization */
void __private loader_init(void);
void __private notifier_init(void);
void __private decode_init(void);
void __private field_init(void);
void __private capture_init(void);

/* Core firestorm API */
void mesg(unsigned char code, const char *fmt, ...) __printf(2,3);
void notify_exit(notifier_t fn, void *priv) __nonull(1);
void firestorm_init(void);
void firestorm_exit(int, const char *, ...) __printf(2,3) __noreturn;

/* Firestorm packet handling API */
void packet_reinject_prepare(struct packet *dst, struct packet *src)
	__nonull(1,2);

#define firestorm_die_error(x) \
	firestorm_exit(EXIT_FAILURE, "%s::%s:%i: " x "(): %s", \
		__FILE__, __FUNCTION__, __LINE__, sys_err2(NULL));

/* Generic vector API */
int vcasecmp(const struct fvec *v1, const struct fvec *v2) __nonull(1,2);
int vcmp(const struct fvec *v1, const struct fvec *v2) __nonull(1,2);
int vstrcmp(const struct fvec *v1, const char *str) __nonull(1,2);
size_t vtouint(struct fvec *v, unsigned int *u) __nonull(1,2);

#endif /* __FIRESTORM_HEADER_INCLUDED__ */
