/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __SIGNATURE_HEADER_INCLUDED__
#define __SIGNATURE_HEADER_INCLUDED__

#include <dotscara/mpool.h>

struct sub_table {
	const char *from;
	const char *to;
};

struct modifier {
	struct modifier *next;
	const char *name;
	char *args;
	int negate;
};

/* A rule criterial in a struct rule */
struct criteria {
	struct matcher *match;
	char *args;
	int negate;
	struct modifier *modifier;
	struct criteria *next;
};

/* A temporary rule structure for parsers */
struct rule {
	struct proto *protocol;

	/* Rule criteria */
	unsigned int num_criteria;
	struct criteria	*criteria;
	struct criteria	*crit_end;

	struct alert	alert;

	struct mpool	crit;
	struct mpool	mod;
};

extern unsigned int signature_count;

struct rule *signature_rule(const char *) __nonull(1);
void signature_abort(struct rule *) __nonull(1);
int signature_criteria(struct rule *, int, const char *, char *) __nonull(1, 3);
int signature_modifier(struct rule *, int, const char *, char *) __nonull(1, 3);
int signature_commit(struct rule *) __nonull(1);
int signature_alert(struct rule *, const char *,
			int, u_int32_t, u_int32_t,
			unsigned long, unsigned long, u_int8_t)
			__nonull(1, 2);
int signature_tag(struct rule *, struct alert_tag *) __nonull(1,2);

#endif /* __SIGNATURE_HEADER_INCLUDED__ */
