/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002,2003,2004 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*/
#include <config.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <firestorm.h>
#include <plugin.h>

#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#endif

#ifdef RTLD_NOW
#define LINK_MODE RTLD_NOW
#else
#define LINK_MODE RTLD_LAZY
#endif

static void firestat(const char *an, const char *fn);
static void firestat(const char *an, const char *fn)
{
	void *lib;
	struct plugin_desc *d;
	char buf[128];

	if ( !(lib=dlopen(fn, LINK_MODE)) ) {
		fprintf(stderr, "%s: %s\n", fn, dlerror());
		return;
	}

	d = dlsym(lib, "firestorm_plugin_desc");

	if ( d == NULL ) {
		fprintf(stderr, "%s: Can't find plugin descriptor\n", fn);
		goto end;
	}

	if ( d->size != sizeof(*d) ) {
		fprintf(stderr, "%s: Invalid plugin descriptor\n", fn);
		goto end;
	}

	buf[0] = '\0';
	if ( dlsym(lib, "firestorm_capdevs") )
		strcat(buf, " packet-capture");
	if ( dlsym(lib, "firestorm_protocols") )
		strcat(buf, " protocols");
	if ( dlsym(lib, "firestorm_matchers") )
		strcat(buf, " matchers");
	if ( dlsym(lib, "firestorm_parsers") )
		strcat(buf, " rule-parsers");
	if ( dlsym(lib, "firestorm_preprocs") )
		strcat(buf, " preprocessors");
	if ( dlsym(lib, "firestorm_targets") )
		strcat(buf, " targets");


	mesg(M_INFO, "Filename: %s", fn);
	mesg(M_INFO, "Summary:  %s", d->desc);
	mesg(M_INFO, "Name:     %s", d->name);
	mesg(M_INFO, "Version:  %i.%i",
		d->ver_major, d->ver_minor);
	mesg(M_INFO, "Author:   %s <%s>",
		d->author_name, d->author_email);
	mesg(M_INFO, "Objects: %s", buf);
	
end:
	dlclose(lib);
	printf("\n");
}

int main(int argc, char **argv)
{
	int i;

	for(i=1; i<argc; i++) {
		firestat(argv[0], argv[i]);
	}

	return 0;
}
