/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002,2003,2004 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
* This is the central registry of all matchers. Usually plugins
* call us when rules are commited to them.
*/
#include <firestorm.h>
#include <loader.h>
#include <alert.h>
#include <signature.h>
#include <string_hash.h>
#include <idx.h>
#include <field.h>
#include <matcher.h>

#define MHASH_SIZE 127
static struct matcher *matchers[MHASH_SIZE];

static void free_nothing(void *foo)
{
}

struct matcher *matcher_find(const char *name)
{
	struct matcher *m;

	for(m=matchers[string_hash(name)%MHASH_SIZE]; m; m=m->next) {
		if ( !strcmp(m->owner->name, name) ) {
			return m;
		}
	}
	
	return NULL;
}

/* We get passed an array of matcher structures, terminated
 * by a NULL element. Add them all to the list */
int matcher_add(struct matcher *m)
{
	u_int32_t h;

	if ( !m->validate )
		return 0;

	h = string_hash(m->owner->name) % MHASH_SIZE;

	if ( !m->free_priv )
		m->free_priv = free_nothing;

	m->next = matchers[h];
	matchers[h] = m;
	return 1;
}
