/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002,2003,2004 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*/
#include <firestorm.h>

#define NOTIFIER_CHUNK 8

struct notifier_chain {
	void *priv;
	notifier_t fn;
	notifier_dtor_t dtor;
};

static struct notifier sysnotify[NOTIFY_MAX];

int notify(int i, notifier_t fn, notifier_dtor_t d, void *p)
{
	if ( i >= NOTIFY_MAX || i < 0 )
		return 0;

	return notifier_add(&sysnotify[i], fn, d, p);
}

void __private notify_run(int i, struct packet *p, int l)
{
	notifier_run(&sysnotify[i], p, l);
}

int notifier_add(struct notifier *n, notifier_t fn, notifier_dtor_t d, void *p)
{
	if ( fn == NULL )
		return 0;

	if ( (n->max - n->num) == 0 ) {
		void *new;
		unsigned short num = n->num + NOTIFIER_CHUNK;
		size_t sz = num * sizeof(*n->chain);

		new = realloc(n->chain, sz);
		if ( new == NULL )
			return 0;

		n->max = num;
		n->chain = new;
	}

	n->chain[n->num].priv = p;
	n->chain[n->num].fn = fn;
	n->chain[n->num].dtor = d;
	n->num++;
	return 1;
}

void notifier_delete(struct notifier *n)
{
	struct notifier_chain *c;
	int i;

	for(c = n->chain, i = 0; i < n->num; i++, c++) {
		if ( c->dtor )
			c->dtor(c->priv);
	}

	free(n->chain);
	n->num = 0;
	n->max = 0;
	n->chain = NULL;
}

void notifier_run(struct notifier *n, struct packet *p, int l)
{
	unsigned short i;

	for(i=0; i < n->num; i++)
		n->chain[i].fn(p, l, n->chain[i].priv);
}

void notifier_run_backwards(struct notifier *n, struct packet *p, int l)
{
	unsigned short i;

	for(i=n->num; i; i--)
		n->chain[i-1].fn(p, l, n->chain[i-1].priv);
}

static void notifier_cleanup(struct packet *pkt, int l, void *priv)
{
	int i;

	for(i=0; i< NOTIFY_MAX; i++)
		notifier_delete(&sysnotify[i]);
}

void __private notifier_init(void)
{
	notify_exit(notifier_cleanup, NULL);
}
