/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2003,2004 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 *
 * TODO:
 *  o Support extensible types
*/
#include <firestorm.h>
#include <dotscara/fobuf.h>
#include <strtouint.h>
#include <idx.h>
#include <i_idx.h>
#include <q_idx.h>
#include <q_filter.h>
#include <q_type.h>

static int str_validate(const char *str, struct q_constraint *c)
{
	if ( c->operator != QF_OP_EQ )
		return 0;

	c->u.op_vec.ptr = (char *)str;
	c->u.op_vec.len = strlen(str);

	return 1;
}

static int int_validate(const char *str, struct q_constraint *c)
{
	int ret;

	ret = strtouint(str, &c->u.op_int.min);
	if ( ret < 0 )
		return 0;

	if ( ret == 0 ) {
		c->u.op_int.max = c->u.op_int.min;
		return 1;
	}

	if ( *(str + ret) != ':' )
		return 0;

	str += ret + 1;

	if ( strtouint(str, &c->u.op_int.max) )
		return 0;

	return 1;
}

static struct q_type builtin_int = {
	.name = "__builtin_integer",
	.type = QFT_INT,
	.validate = int_validate,
};

static struct q_type builtin_str = {
	.name = "__builtin_string",
	.type = QFT_STRING,
	.validate = str_validate,
};

/* q_type_compatible:
 * @t: q_type
 * @type: underlying database type
 *
 * Determine if an underlying database type is compatible with the type
 * contained in the query.
 *
 * Return value: 1 if they are compatible, 0 if not.
*/
int q_type_compatible(struct q_type *t, int type)
{
	if ( t == &builtin_int ) {
		if ( type == IDX_SKUNK || type == IDX_SKUNK_NOCASE )
			return 0;
		return 1;
	}

	if ( t == &builtin_str ) {
		if ( type != IDX_SKUNK && type != IDX_SKUNK_NOCASE)
			return 0;
		return 1;
	}

	return ( t->underlying_type == type );
}

/* q_type_get:
 * @field: Field name
 * @val: Value, for probing to guess the type.
 *
 * Figure out what type a particular field is.
 *
 * Return value: pointer to the type, or NULL on error.
 */
struct q_type *q_type_get(const char *field, const char *val)
{
	if ( isdigit(*val) )
		return &builtin_int;
	else
		return &builtin_str;
}

